from unittest.mock import AsyncMock

import pytest
from alist_mikananirss.extractor import Extractor, ExtractorBase, RegexExtractor


@pytest.fixture
def mock_extractor():
    return AsyncMock(spec=ExtractorBase)


@pytest.fixture
def mock_regex_extractor():
    return AsyncMock(spec=RegexExtractor)


def test_initialize():
    mock_extractor = AsyncMock(spec=ExtractorBase)
    Extractor.initialize(mock_extractor)
    assert Extractor._extractor == mock_extractor


def test_get_instance_without_initialization():
    Extractor._instance = None
    with pytest.raises(ValueError):
        Extractor.get_instance()


def test_get_instance_after_initialization(mock_extractor):
    Extractor.initialize(mock_extractor)
    instance = Extractor.get_instance()
    assert isinstance(instance, Extractor)


def test_multiple_initializations():
    # Test behavior with multiple initializations
    mock_extractor1 = AsyncMock(spec=ExtractorBase)
    mock_extractor2 = AsyncMock(spec=ExtractorBase)

    Extractor.initialize(mock_extractor1)
    first_instance = Extractor.get_instance()

    Extractor.initialize(mock_extractor2)
    second_instance = Extractor.get_instance()

    assert first_instance is second_instance
    assert Extractor._extractor == mock_extractor2


@pytest.mark.asyncio
async def test_analyse_resource_title_without_initialization():
    # Test analyse_resource_title when not initialized
    Extractor._instance = None
    Extractor._extractor = None

    with pytest.raises(ValueError):
        await Extractor.analyse_resource_title("Test")



import os
from unittest.mock import AsyncMock, MagicMock, patch

import pytest
import yaml
from alist_mikananirss.bot import PushPlusBot, TelegramBot
from alist_mikananirss.common import initializer
from alist_mikananirss.core import RegexFilter, RssMonitor

# 模拟的YAML配置文件内容
MOCK_YAML_CONTENT = """
common:
  interval_time: 300
  proxies:
    http: https://127.0.0.1:7890
    https: https://127.0.0.1:7890

alist:
  base_url: https://www.example.com
  token: alist-xxx
  downloader: aria2
  download_path: Onedrive/Anime

mikan:
  subscribe_url: 
    - https://mikanani.me/RSS/MyBangumi?token=xxx
  regex_pattern:
    简体: "(简体|简中|简日|CHS)"
    1080p: "(X1080|1080P)"
  filters:
    - 1080p

notification:
  telegram:
    bot_token: your_token
    user_id: your_id
  pushplus: 
    token: xxxxx
  interval_time: 300

rename: 
  chatgpt: 
    api_key: sk-xxx
    base_url: https://example.com/v1
    model: gpt-3.5-turbo
  rename_format: "{name} S{season:02d}E{episode:02d}"

dev:
  log_level: INFO
"""


@pytest.fixture(scope="module")
def temp_config_file():
    config_path = "temp_config.yaml"
    with open(config_path, "w", encoding="utf-8") as f:
        f.write(MOCK_YAML_CONTENT)
    yield config_path
    os.remove(config_path)


@pytest.fixture(autouse=True)
def setup_config(temp_config_file):
    initializer.read_config(temp_config_file)
    yield
    initializer.config_loader = None


def test_read_config(temp_config_file):
    initializer.read_config(temp_config_file)
    assert initializer.config_loader is not None


@patch("alist_mikananirss.common.initializer.logger")
def test_setup_logger(mock_logger):
    initializer.setup_logger()
    mock_logger.remove.assert_called_once()
    mock_logger.add.assert_called()


def test_setup_proxy():
    with patch.dict(os.environ, {}, clear=True):
        initializer.setup_proxy()
        assert os.environ.get("HTTP_PROXY") == "https://127.0.0.1:7890"
        assert os.environ.get("HTTPS_PROXY") == "https://127.0.0.1:7890"


@pytest.mark.asyncio
async def test_init_alist():
    with patch("alist_mikananirss.common.initializer.Alist") as MockAlist:
        mock_alist = MockAlist.return_value
        mock_alist.get_alist_ver = AsyncMock(return_value="3.29.0")
        result = await initializer.init_alist()
        assert isinstance(result, MagicMock)
        MockAlist.assert_called_once()


def test_init_extrator():
    with patch(
        "alist_mikananirss.common.initializer.ChatGPTExtractor"
    ) as MockChatGPTExtractor:
        initializer.init_extrator()
        MockChatGPTExtractor.assert_called_once_with(
            "sk-xxx", "https://example.com/v1", "gpt-3.5-turbo"
        )


def test_init_notification_bots():
    result = initializer.init_notification_bots()
    assert len(result) == 2
    assert isinstance(result[0].bot, TelegramBot)
    assert result[0].bot.bot_token == "your_token"
    assert result[0].bot.user_id == "your_id"
    assert isinstance(result[1].bot, PushPlusBot)
    assert result[1].bot.user_token == "xxxxx"
    assert result[1].bot.channel.value == "wechat"


def test_init_notification_sender():
    with patch(
        "alist_mikananirss.common.initializer.NotificationSender"
    ) as MockNotificationSender:
        initializer.init_notification_sender()
        MockNotificationSender.initialize.assert_called_once()


def test_init_resource_filter():
    result = initializer.init_resource_filter()
    assert isinstance(result, RegexFilter)
    assert len(result.patterns) == 1


def test_init_rss_monitor():
    regex_filter = RegexFilter()
    result = initializer.init_rss_monitor(regex_filter)
    assert isinstance(result, RssMonitor)
    assert result.interval_time == 300


def test_init_download_manager():
    with patch(
        "alist_mikananirss.common.initializer.DownloadManager"
    ) as MockDownloadManager:
        mock_alist = MagicMock()
        initializer.init_download_manager(mock_alist)
        MockDownloadManager.initialize.assert_called_once_with(
            mock_alist, "Onedrive/Anime", True, True
        )


def test_init_renamer():
    with patch("alist_mikananirss.common.initializer.AnimeRenamer") as MockAnimeRenamer:
        mock_alist = MagicMock()
        initializer.init_renamer(mock_alist)
        MockAnimeRenamer.initialize.assert_called_once_with(
            mock_alist, "{name} S{season:02d}E{episode:02d}"
        )


def test_missing_optional_configs(temp_config_file):
    yaml_without_optional = yaml.safe_load(MOCK_YAML_CONTENT)
    del yaml_without_optional["notification"]
    del yaml_without_optional["rename"]

    with open(temp_config_file, "w", encoding="utf-8") as f:
        yaml.dump(yaml_without_optional, f)

    initializer.read_config(temp_config_file)

    result = initializer.init_notification_bots()
    assert len(result) == 0

    with patch(
        "alist_mikananirss.common.initializer.DownloadManager"
    ) as MockDownloadManager:
        mock_alist = MagicMock()
        initializer.init_download_manager(mock_alist)
        MockDownloadManager.initialize.assert_called_once_with(
            mock_alist, "Onedrive/Anime", False, False
        )


@pytest.mark.asyncio
async def test_missing_required_configs(temp_config_file):
    yaml_without_required = yaml.safe_load(MOCK_YAML_CONTENT)
    del yaml_without_required["alist"]

    with open(temp_config_file, "w", encoding="utf-8") as f:
        yaml.dump(yaml_without_required, f)
    initializer.read_config(temp_config_file)

    with pytest.raises(KeyError):
        await initializer.init_alist()


def test_invalid_configs(temp_config_file):
    yaml_with_invalid = yaml.safe_load(MOCK_YAML_CONTENT)
    yaml_with_invalid["common"]["interval_time"] = -1

    with open(temp_config_file, "w", encoding="utf-8") as f:
        yaml.dump(yaml_with_invalid, f)

    initializer.read_config(temp_config_file)

    with pytest.raises(ValueError):
        initializer.init_rss_monitor(RegexFilter())



import asyncio
from unittest.mock import Mock

import pytest
from alist_mikananirss.bot import NotificationBot
from alist_mikananirss.core import NotificationSender
from alist_mikananirss.websites import ResourceInfo


@pytest.fixture
def reset_notification_sender():
    NotificationSender._instance = None
    yield
    NotificationSender._instance = None


@pytest.mark.asyncio
async def test_initialization(reset_notification_sender):
    mock_bot = Mock(spec=NotificationBot)
    NotificationSender.initialize([mock_bot], interval=30)

    assert NotificationSender._instance is not None
    assert len(NotificationSender._instance.notification_bots) == 1
    assert NotificationSender._instance._interval == 30
    assert isinstance(NotificationSender._instance._queue, asyncio.Queue)


@pytest.mark.asyncio
async def test_get_instance_without_initialization():
    with pytest.raises(RuntimeError):
        NotificationSender.get_instance()


@pytest.mark.asyncio
async def test_set_notification_bots(reset_notification_sender):
    NotificationSender.initialize([])
    mock_bot1 = Mock(spec=NotificationBot)
    mock_bot2 = Mock(spec=NotificationBot)

    NotificationSender.set_notification_bots([mock_bot1, mock_bot2])

    assert len(NotificationSender._instance.notification_bots) == 2


@pytest.mark.asyncio
async def test_add_resource(reset_notification_sender):
    NotificationSender.initialize([])
    resource = ResourceInfo(
        anime_name="test name",
        resource_title="Test",
        torrent_url="https://test.com",
        published_date="2023-01-01",
    )

    await NotificationSender.add_resource(resource)

    assert NotificationSender._instance._queue.qsize() == 1


@pytest.mark.asyncio
async def test_set_interval(reset_notification_sender):
    NotificationSender.initialize([], interval=60)

    NotificationSender.set_interval(120)

    assert NotificationSender._instance._interval == 120


@pytest.mark.asyncio
async def test_run_method(reset_notification_sender):
    mock_bot = Mock(spec=NotificationBot)
    mock_bot.send_message.return_value = asyncio.Future()
    mock_bot.send_message.return_value.set_result(None)

    NotificationSender.initialize([mock_bot], interval=0.1)
    resource = ResourceInfo(
        anime_name="test name",
        resource_title="Test",
        torrent_url="https://test.com",
        published_date="2023-01-01",
    )
    await NotificationSender.add_resource(resource)

    # Allow some time for the _run method to execute
    await asyncio.sleep(0.2)

    mock_bot.send_message.assert_called_once()


@pytest.mark.asyncio
async def test_send_method_with_exception(reset_notification_sender):
    mock_bot1 = Mock(spec=NotificationBot)
    mock_bot1.send_message.return_value = asyncio.Future()
    mock_bot1.send_message.return_value.set_result(None)

    mock_bot2 = Mock(spec=NotificationBot)
    mock_bot2.send_message.side_effect = Exception("Test exception")

    NotificationSender.initialize([mock_bot1, mock_bot2], interval=0.1)
    resource = ResourceInfo(
        anime_name="test name",
        resource_title="Test",
        torrent_url="https://test.com",
        published_date="2023-01-01",
    )
    await NotificationSender.add_resource(resource)

    await asyncio.sleep(0.2)

    mock_bot1.send_message.assert_called_once()
    mock_bot2.send_message.assert_called_once()


@pytest.mark.asyncio
async def test_send_method_without_bots(reset_notification_sender):
    NotificationSender.initialize([], interval=0.1)
    resource = ResourceInfo(
        anime_name="test name",
        resource_title="Test",
        torrent_url="https://test.com",
        published_date="2023-01-01",
    )
    await NotificationSender.add_resource(resource)

    await asyncio.sleep(0.2)



import asyncio

from loguru import logger

from alist_mikananirss.common.database import SubscribeDatabase
from alist_mikananirss.websites import ResourceInfo, WebsiteFactory

from .download_manager import DownloadManager
from .filters import RegexFilter


class RssMonitor:
    def __init__(
        self,
        subscribe_urls: list[str] | str,
        filter: RegexFilter,
        use_extractor: bool = False,
    ) -> None:
        """The rss feed manager"""
        if not isinstance(subscribe_urls, list):
            subscribe_urls = [subscribe_urls]
        self.subscribe_urls = subscribe_urls
        self.websites = [
            WebsiteFactory.get_website_parser(url) for url in subscribe_urls
        ]
        self.filter = filter
        self.use_extractor = use_extractor
        self.db = SubscribeDatabase()
        self.interval_time = 300

    def set_interval_time(self, interval_time: int):
        self.interval_time = interval_time

    async def get_new_resources(self, m_filter: RegexFilter) -> list[ResourceInfo]:
        "Parse all rss url and get the filtered, unique resource info list"

        async def process_entry(self, website, entry):
            "get resource info from feed entry"
            try:
                resource_info = await website.extract_resource_info(
                    entry, self.use_extractor
                )
            except Exception as e:
                logger.error(f"Pass {entry.resource_title} because of error: {e}")
                return None
            return resource_info

        new_resources_set: set[ResourceInfo] = set()
        tasks = []
        for website in self.websites:
            feed_entries = await website.get_feed_entries()
            feed_entries_filted = filter(
                lambda entry: m_filter.filt_single(entry.resource_title),
                feed_entries,
            )
            for entry in feed_entries_filted:
                if not self.db.is_resource_title_exist(entry.resource_title):
                    task = asyncio.create_task(process_entry(self, website, entry))
                    tasks.append(task)
        results = await asyncio.gather(*tasks, return_exceptions=True)
        for result in results:
            if result:
                new_resources_set.add(result)
        new_resources = list(new_resources_set)
        return new_resources

    async def run(self):
        while 1:
            logger.info("Start update checking")
            new_resources = await self.get_new_resources(self.filter)
            if not new_resources:
                logger.info("No new resources")
            else:
                await DownloadManager.add_download_tasks(new_resources)
            await asyncio.sleep(self.interval_time)



import argparse
import asyncio

from alist_mikananirss.common import initializer


async def run():
    parser = argparse.ArgumentParser(description="Alist Mikanani RSS")
    parser.add_argument(
        "--config",
        default="config.yaml",
        help="Path to the configuration file",
    )

    args = parser.parse_args()

    initializer.read_config(args.config)
    initializer.setup_logger()
    initializer.setup_proxy()
    alist_client = await initializer.init_alist()
    initializer.init_download_manager(alist_client)
    initializer.init_extrator()
    initializer.init_renamer(alist_client)
    resource_filters = initializer.init_resource_filter()
    rss_monitor = initializer.init_rss_monitor(resource_filters)
    initializer.init_notification_sender()

    task = asyncio.create_task(rss_monitor.run())
    await task


def main():
    asyncio.run(run())



from __future__ import annotations

import re
from abc import ABC, abstractmethod
from dataclasses import dataclass
from enum import Enum
from typing import Optional, Union

from loguru import logger


# https://github.com/alist-org/alist/blob/86b35ae5cfec400871072356fec4dea88303195d/pkg/task/task.go#L27
class AlistTaskStatus(Enum):
    Pending = 0
    Running = 1
    Succeeded = 2
    Canceling = 3
    Canceled = 4
    Errored = 5
    Failing = 6
    Failed = 7
    StateWaitingRetry = 8
    StateBeforeRetry = 9
    UNKNOWN = 10


class AlistDownloaderType(Enum):
    ARIA = "aria2"
    QBIT = "qBittorrent"


class AlistDeletePolicy(Enum):
    DeleteOnUploadSucceed = "delete_on_upload_succeed"
    DeleteOnUploadFailed = "delete_on_upload_failed"
    DeleteNever = "delete_never"
    DeleteAlways = "delete_always"


class AlistTaskType(Enum):
    DOWNLOAD = "offline_download"
    TRANSFER = "offline_download_transfer"
    UNKNOWN = "unknown"


# for task related api call
class AlistTaskState(Enum):
    DONE = "done"
    UNDONE = "undone"


DOWNLOAD_DES_PATTERN = re.compile(r"download\s+(.+?)\s+to")
TRANSFER_DES_PATTERN = re.compile(r"transfer (.+?) to \[")


class AlistTaskError(Exception):
    """Base exception for AlistTask related errors."""


class InvalidTaskDescription(AlistTaskError):
    """Raised when task description is invalid."""


@dataclass
class AlistTask(ABC):
    tid: str
    description: str
    status: AlistTaskStatus
    progress: float
    error_msg: Optional[str] = None
    task_type: AlistTaskType = AlistTaskType.UNKNOWN

    @classmethod
    @abstractmethod
    def from_json(cls, json_data: dict) -> AlistTask:
        tid = json_data["id"]
        description = json_data["name"]
        state_str = json_data["state"]
        progress = json_data["progress"]
        error_str = json_data["error"]
        try:
            status = AlistTaskStatus(state_str)
        except ValueError:
            logger.warning(f"Unknown task status {state_str} of task {tid}")
            status = AlistTaskStatus.UNKNOWN
        return cls(tid, description, status, progress, error_str)


@dataclass
class AlistTransferTask(AlistTask):
    uuid: str = ""
    file_name: str = ""

    def __post_init__(self):
        self.task_type = AlistTaskType.TRANSFER

    @classmethod
    def from_json(cls, json_data: dict) -> AlistTransferTask:
        task = super().from_json(json_data)
        match = re.search(TRANSFER_DES_PATTERN, task.description)
        if match:
            extracted_string = match.group(1)
            elements = extracted_string.split("/")
            uuid = elements[-2]
            file_name = elements[-1]
        else:
            raise InvalidTaskDescription(
                f"Failed to get uuid and file name from task description: {task.description}"
            )

        _instance = cls(
            tid=task.tid,
            description=task.description,
            status=task.status,
            progress=task.progress,
            error_msg=task.error_msg,
            uuid=uuid,
            file_name=file_name,
        )
        return _instance


@dataclass
class AlistDownloadTask(AlistTask):
    url: str = ""

    def __post_init__(self):
        self.task_type = AlistTaskType.DOWNLOAD

    @classmethod
    def from_json(cls, json_data: dict) -> AlistDownloadTask:
        task = super().from_json(json_data)
        match = re.match(DOWNLOAD_DES_PATTERN, task.description)
        if match:
            url = match.group(1)
        else:
            raise InvalidTaskDescription(
                f"Failed to get url from task description: {task.description}"
            )
        _instance = cls(
            tid=task.tid,
            description=task.description,
            status=task.status,
            progress=task.progress,
            error_msg=task.error_msg,
            url=url,
        )
        return _instance


class AlistTaskCollection:
    def __init__(
        self, tasks_list: list[Union[AlistTransferTask, AlistDownloadTask]] = None
    ):
        self.tasks = {}
        if tasks_list:
            self.tasks = {task.tid: task for task in tasks_list}

    def add_task(self, task: Union[AlistTransferTask, AlistDownloadTask]) -> None:
        self.tasks[task.tid] = task

    def __add__(self, other: "AlistTaskCollection") -> "AlistTaskCollection":
        if not isinstance(other, AlistTaskCollection):
            raise TypeError("Operand must be an instance of AlistTaskCollection")
        new_tasks = self.tasks.copy()
        new_tasks.update(other.tasks)
        return AlistTaskCollection(list(new_tasks.values()))

    def __getitem__(
        self, index: Union[int, str]
    ) -> Optional[Union[AlistDownloadTask, AlistTransferTask]]:
        if isinstance(index, int):
            return list(self.tasks.values())[index]
        return self.tasks.get(index)

    def __contains__(self, task: AlistTask) -> bool:
        return task.tid in self.tasks

    def __len__(self) -> int:
        return len(self.tasks)

    def __iter__(self):
        return iter(self.tasks.values())



from .base import ExtractorBase
from .models import AnimeNameExtractResult, ResourceTitleExtractResult
from .regex import RegexExtractor


class Extractor:
    _instance = None
    _extractor = None
    _tmp_regex_extractor = None

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super(Extractor, cls).__new__(cls)
        return cls._instance

    @classmethod
    def initialize(cls, extractor: ExtractorBase):
        """Initialize the Extractor with a specific extractor."""
        if cls._instance is None:
            cls._instance = cls()
        cls._extractor = extractor
        cls._tmp_regex_extractor = RegexExtractor()

    @classmethod
    def get_instance(cls):
        """Get the singleton instance of Extractor."""
        if cls._instance is None:
            raise ValueError(
                "Extractor not initialized. Please use initialize() first."
            )
        return cls._instance

    @classmethod
    async def analyse_anime_name(cls, anime_name: str) -> AnimeNameExtractResult:
        # chatgpt对番剧名分析不稳定，所以固定用正则分析番剧名
        instance = cls.get_instance()
        return await instance._tmp_regex_extractor.analyse_anime_name(anime_name)

    @classmethod
    async def analyse_resource_title(
        cls, resource_name: str
    ) -> ResourceTitleExtractResult:
        instance = cls.get_instance()
        return await instance._extractor.analyse_resource_title(resource_name)



from unittest.mock import AsyncMock, MagicMock, patch

import feedparser
import pytest
from alist_mikananirss.websites import FeedEntry, ResourceInfo
from alist_mikananirss.websites.mikan import Mikan, MikanHomePageInfo


@pytest.fixture
def mikan():
    return Mikan("https://mikanani.me/RSS/Bangumi?bangumiId=2742&subgroupid=370")


@pytest.fixture
def mock_rss_data():
    return """<rss version="2.0">
<channel>
<title>Mikan Project - GIRLS BAND CRY</title>
<link>https://mikanani.me/RSS/Bangumi?bangumiId=3298&subgroupid=382</link>
<description>Mikan Project - GIRLS BAND CRY</description>
<item>
<guid isPermaLink="false">【喵萌Production】★04月新番★[GIRLS BAND CRY][01-13][1080p][繁日双语][招募翻译时轴]</guid>
<link>https://mikanani.me/Home/Episode/a19d5da34e2ec205bddd9c6935ab579ff37da7d7</link>
<title>【喵萌Production】★04月新番★[GIRLS BAND CRY][01-13][1080p][繁日双语][招募翻译时轴]</title>
<description>【喵萌Production】★04月新番★[GIRLS BAND CRY][01-13][1080p][繁日双语][招募翻译时轴][8.8GB]</description>
<torrent xmlns="https://mikanani.me/0.1/">
<link>https://mikanani.me/Home/Episode/a19d5da34e2ec205bddd9c6935ab579ff37da7d7</link>
<contentLength>9448928256</contentLength>
<pubDate>2024-07-17T19:09:00</pubDate>
</torrent>
<enclosure type="application/x-bittorrent" length="9448928256" url="https://mikanani.me/Download/20240717/a19d5da34e2ec205bddd9c6935ab579ff37da7d7.torrent"/>
</item>
</channel>
</rss>"""


@pytest.mark.asyncio
async def test_get_feed_entries(mikan, mock_rss_data):
    with patch.object(
        mikan, "parse_feed", return_value=feedparser.parse(mock_rss_data)
    ):
        result = await mikan.get_feed_entries()

    assert isinstance(result, list)
    assert len(result) == 1
    entry = result.pop()
    assert isinstance(entry, FeedEntry)
    assert (
        entry.resource_title
        == "【喵萌Production】★04月新番★[GIRLS BAND CRY][01-13][1080p][繁日双语][招募翻译时轴]"
    )
    assert (
        entry.torrent_url
        == "https://mikanani.me/Download/20240717/a19d5da34e2ec205bddd9c6935ab579ff37da7d7.torrent"
    )
    assert entry.published_date == "2024-07-17T19:09:00"
    assert (
        entry.homepage_url
        == "https://mikanani.me/Home/Episode/a19d5da34e2ec205bddd9c6935ab579ff37da7d7"
    )


@pytest.mark.asyncio
@patch("aiohttp.ClientSession.get")
async def test_parse_homepage(mock_get, mikan):
    # Setup mock response
    mock_response = AsyncMock()
    mock_response.raise_for_status = MagicMock()
    mock_response.text = AsyncMock(
        return_value="""
    <html>
        <p class="bangumi-title">GIRLS BAND CRY</p>
        <p class="bangumi-info">字幕组：喵萌奶茶屋</p>
    </html>
    """
    )
    mock_get.return_value.__aenter__.return_value = mock_response
    result = await mikan.parse_homepage(
        "https://mikanani.me/Home/Episode/a19d5da34e2ec205bddd9c6935ab579ff37da7d7"
    )
    assert isinstance(result, MikanHomePageInfo)
    assert result.anime_name == "GIRLS BAND CRY"
    assert result.fansub == "喵萌奶茶屋"


@pytest.mark.asyncio
async def test_extract_resource_info(mikan):
    mock_entry = FeedEntry(
        resource_title="【喵萌Production】★04月新番★[GIRLS BAND CRY][01-13][1080p][繁日双语][招募翻译时轴]",
        torrent_url="https://mikanani.me/Download/20240717/a19d5da34e2ec205bddd9c6935ab579ff37da7d7.torrent",
        published_date="2024-07-17T19:09:00",
        homepage_url="https://mikanani.me/Home/Episode/a19d5da34e2ec205bddd9c6935ab579ff37da7d7",
    )

    mock_homepage_info = MikanHomePageInfo(
        anime_name="GIRLS BAND CRY", fansub="喵萌奶茶屋"
    )

    with patch.object(mikan, "parse_homepage", return_value=mock_homepage_info):
        result = await mikan.extract_resource_info(mock_entry)

    assert isinstance(result, ResourceInfo)
    assert result.anime_name == "GIRLS BAND CRY"
    assert (
        result.resource_title
        == "【喵萌Production】★04月新番★[GIRLS BAND CRY][01-13][1080p][繁日双语][招募翻译时轴]"
    )
    assert (
        result.torrent_url
        == "https://mikanani.me/Download/20240717/a19d5da34e2ec205bddd9c6935ab579ff37da7d7.torrent"
    )
    assert result.published_date == "2024-07-17T19:09:00"
    assert result.fansub == "喵萌奶茶屋"



from unittest.mock import AsyncMock, MagicMock, patch

import pytest
from alist_mikananirss.alist import Alist
from alist_mikananirss.alist.tasks import (
    AlistDownloadTask,
    AlistTaskCollection,
    AlistTaskStatus,
    AlistTransferTask,
)
from alist_mikananirss.common.database import SubscribeDatabase
from alist_mikananirss.core import DownloadManager
from alist_mikananirss.core.download_manager import AnimeDownloadTaskInfo, TaskMonitor
from alist_mikananirss.websites import ResourceInfo
from loguru import logger


@pytest.fixture
def mock_alist():
    return AsyncMock(spec=Alist)


@pytest.fixture
def mock_db():
    return MagicMock(spec=SubscribeDatabase)


@pytest.fixture
def download_manager(mock_alist, mock_db):
    DownloadManager.initialize(
        mock_alist, "/base/path", use_renamer=True, need_notification=True
    )
    DownloadManager._instance.db = mock_db
    return DownloadManager.get_instance()


@pytest.fixture
def test_rousources():
    resources = [
        ResourceInfo(
            resource_title="title1",
            torrent_url="https://test1.torrent",
            published_date="1",
            anime_name="Test Anime",
            season=1,
            episode=5,
            fansub="TestSub",
            quality="1080p",
            language="JP",
        ),
        ResourceInfo(
            resource_title="title2",
            torrent_url="https://test2.torrent",
            published_date="1",
            anime_name="Test Anime2",
            season=1,
            episode=5,
            fansub="TestSub",
            quality="1080p",
            language="JP",
        ),
    ]
    return resources


def test_initialize():
    with patch(
        "alist_mikananirss.common.database.SubscribeDatabase"
    ) as MockSubscribeDatabase:
        alist = AsyncMock()
        db_mock = MagicMock()
        MockSubscribeDatabase.side_effect = db_mock

        DownloadManager.initialize(alist, "test/path", True)
        assert DownloadManager.get_instance().alist_client == alist
        assert DownloadManager.get_instance().base_download_path == "test/path"
        assert DownloadManager.get_instance().use_renamer
        assert not DownloadManager.get_instance().need_notification


@pytest.mark.asyncio
async def test_download(download_manager, mock_alist, test_rousources):
    mock_alist.add_offline_download_task.side_effect = [
        AlistTaskCollection(
            [
                AlistDownloadTask(
                    tid=1,
                    description="Download task 1",
                    status="success",
                    progress=100,
                    error_msg=None,
                    url=test_rousources[0].torrent_url,
                ),
            ]
        ),
        TimeoutError,
    ]
    with patch.object(logger, "error") as logger_error_mock:
        success_task_info = await download_manager.download(test_rousources)
        assert len(success_task_info) == 1
        assert success_task_info[0].resource == test_rousources[0]
        logger_error_mock.assert_called_once()


@pytest.mark.asyncio
async def test_monitor_success(download_manager, mock_db):

    test_task = AnimeDownloadTaskInfo(
        resource=ResourceInfo(
            resource_title="title",
            torrent_url="https://test1.torrent",
            published_date="1",
            anime_name="Test Anime",
            season=1,
            episode=5,
            fansub="TestSub",
            quality="1080p",
            language="JP",
        ),
        download_path="test/path",
        download_task=MagicMock(),
    )
    with (
        patch.object(download_manager, "_wait_finished") as wait_finished_mock,
        patch.object(download_manager, "_post_process") as post_process_mock,
    ):
        wait_finished_mock.return_value = MagicMock()
        await download_manager.monitor(test_task)
        wait_finished_mock.assert_awaited_once_with(test_task)
        post_process_mock.assert_called_once()
        assert mock_db.delete_by_resource_title.call_count == 0


@pytest.mark.asyncio
async def test_monitor_failed(download_manager, mock_db):
    test_task = AnimeDownloadTaskInfo(
        resource=ResourceInfo(
            resource_title="title",
            torrent_url="https://test1.torrent",
            published_date="1",
            anime_name="Test Anime",
            season=1,
            episode=5,
            fansub="TestSub",
            quality="1080p",
            language="JP",
        ),
        download_path="test/path",
        download_task=MagicMock(),
    )
    with (
        patch.object(download_manager, "_wait_finished") as wait_finished_mock,
        patch.object(download_manager, "_post_process") as post_process_mock,
    ):
        wait_finished_mock.return_value = None
        await download_manager.monitor(test_task)
        wait_finished_mock.assert_awaited_once_with(test_task)
        assert post_process_mock.call_count == 0
        mock_db.delete_by_resource_title.assert_called_once_with(
            test_task.resource.resource_title
        )


@pytest.mark.asyncio
async def test_add_download_task(download_manager, mock_db, test_rousources):
    download_tasks_info = [
        AnimeDownloadTaskInfo(
            resource=test_rousources[0],
            download_path="test/path",
            download_task=MagicMock(),
        ),
        AnimeDownloadTaskInfo(
            resource=test_rousources[1],
            download_path="test/path",
            download_task=MagicMock(),
        ),
    ]
    with patch.object(download_manager, "download") as download_mock:
        with patch.object(download_manager, "monitor") as monitor_mock:
            download_mock.return_value = download_tasks_info
            await DownloadManager.add_download_tasks(test_rousources)
            download_mock.assert_called_once()
            assert mock_db.insert_resource_info.call_count == 2
            assert monitor_mock.call_count == 2


@pytest.mark.asyncio
async def test_find_transfer_task(download_manager):
    resource = ResourceInfo(
        anime_name="Test Anime",
        resource_title="Episode 1",
        torrent_url="https://example.com/test.torrent",
    )
    transfer_task = AlistTransferTask(
        tid="123",
        description="transfer Test Anime/file.mp4 to [/path]",
        status=AlistTaskStatus.Running,
        progress=50,
        uuid="123",
        file_name="file.mp4",
    )

    download_manager.alist_client.get_task_list.return_value = [transfer_task]

    with patch("asyncio.sleep", new_callable=AsyncMock):
        result = await download_manager._find_transfer_task(resource)

    assert result == transfer_task
    assert transfer_task.uuid in download_manager.uuid_set


@pytest.mark.asyncio
async def test_wait_finished_success(download_manager):
    resource = ResourceInfo(
        anime_name="Test Anime",
        resource_title="Episode 1",
        torrent_url="https://example.com/test.torrent",
    )
    download_task = AlistDownloadTask(
        tid="123",
        description="download https://example.com/test.torrent to",
        status=AlistTaskStatus.Running,
        progress=0.5,
        url="https://example.com/test.torrent",
    )
    transfer_task = AlistTransferTask(
        tid="456",
        description="transfer Test Anime/file.mp4 to [/path]",
        status=AlistTaskStatus.Running,
        progress=0.5,
        uuid="uuid",
        file_name="file.mp4",
    )

    task_info = AnimeDownloadTaskInfo(
        resource=resource,
        download_path="/base/path/Test Anime",
        download_task=download_task,
    )
    with (
        patch(
            "alist_mikananirss.core.download_manager.TaskMonitor"
        ) as mock_task_monitor,
        patch.object(
            download_manager,
            "_find_transfer_task",
            AsyncMock(return_value=transfer_task),
        ),
    ):
        mock_download_task_monitor = AsyncMock(spec=TaskMonitor)
        mock_download_task_monitor.wait_finished.return_value = AlistDownloadTask(
            tid="123",
            description="",
            status=AlistTaskStatus.Succeeded,
            progress=1.0,
            url="https://example.com/test.torrent",
        )
        mock_transfer_task_monitor = MagicMock(spec=TaskMonitor)
        mock_transfer_task_monitor.wait_finished.return_value = AlistTransferTask(
            tid="456",
            description="",
            status=AlistTaskStatus.Succeeded,
            progress=1.0,
            uuid="uuid",
            file_name="file.mp4",
        )
        mock_task_monitor.side_effect = [
            mock_download_task_monitor,
            mock_transfer_task_monitor,
        ]
        result = await download_manager._wait_finished(task_info)
        assert result is not None
        assert result.transfer_task.tid == transfer_task.tid



import math
from unittest.mock import AsyncMock, MagicMock, patch

import pytest
from alist_mikananirss.alist.api import Alist, AlistConfig
from alist_mikananirss.alist.tasks import (
    AlistDeletePolicy,
    AlistDownloaderType,
    AlistDownloadTask,
    AlistTaskCollection,
    AlistTaskState,
    AlistTaskStatus,
    AlistTaskType,
    AlistTransferTask,
)


@pytest.fixture
def alist_config():
    return AlistConfig(
        base_url="https://example.com",
        token="test_token",
        downloader=AlistDownloaderType.ARIA,
    )


@pytest.fixture
def alist(alist_config):
    return Alist(alist_config)


@pytest.mark.asyncio
async def test_alist_init(alist):
    assert alist.config.base_url == "https://example.com"
    assert alist.config.token == "test_token"
    assert alist.config.downloader == AlistDownloaderType.ARIA


@pytest.mark.asyncio
async def test_get_alist_ver(alist):
    with patch.object(alist, "_api_call", new_callable=AsyncMock) as mock_api_call:
        mock_api_call.return_value = {"version": "v2.0.0"}
        version = await alist.get_alist_ver()
        assert version == "2.0.0"
        mock_api_call.assert_called_once_with("GET", "/api/public/settings")


@pytest.mark.asyncio
async def test_add_offline_download_task(alist):
    with patch.object(alist, "_api_call", new_callable=AsyncMock) as mock_api_call:
        mock_api_call.return_value = {
            "tasks": [
                {
                    "id": "task1",
                    "name": "download https://example.com/file.zip to /path/to/save",
                    "state": 0,
                    "progress": 0.0,
                    "error": None,
                }
            ]
        }
        tasks = await alist.add_offline_download_task(
            "/path/to/save", ["https://example.com/file.zip"]
        )
        assert isinstance(tasks, AlistTaskCollection)
        assert len(tasks) == 1
        assert isinstance(tasks[0], AlistDownloadTask)
        assert tasks[0].tid == "task1"
        assert tasks[0].url == "https://example.com/file.zip"
        mock_api_call.assert_called_once_with(
            "POST",
            "api/fs/add_offline_download",
            json={
                "delete_policy": AlistDeletePolicy.DeleteAlways.value,
                "path": "/path/to/save",
                "urls": ["https://example.com/file.zip"],
                "tool": AlistDownloaderType.ARIA,
            },
        )


@pytest.mark.asyncio
async def test_upload(alist):
    with (
        patch.object(alist, "_api_call", new_callable=AsyncMock) as mock_api_call,
        patch("os.path.abspath", return_value="/local/path/file.txt"),
        patch("os.stat") as mock_stat,
        patch("builtins.open", create=True) as mock_open,
        patch("mimetypes.guess_type", return_value=("application/octet-stream", None)),
    ):

        mock_stat.return_value = MagicMock(st_size=1024)
        mock_file = MagicMock()
        mock_open.return_value.__enter__.return_value = mock_file

        result = await alist.upload("/remote/path", "/local/path/file.txt")

        assert result is True
        mock_api_call.assert_called_once()
        args, kwargs = mock_api_call.call_args
        assert args[0] == "PUT"
        assert args[1] == "api/fs/put"
        assert kwargs["custom_headers"]["file-path"] == "/remote/path/file.txt"
        assert kwargs["custom_headers"]["Content-Type"] == "application/octet-stream"
        assert kwargs["data"] == mock_file


@pytest.mark.asyncio
async def test_list_dir(alist):
    with patch.object(alist, "_api_call", new_callable=AsyncMock) as mock_api_call:
        mock_api_call.return_value = {
            "content": [{"name": "file1.txt"}, {"name": "file2.txt"}]
        }
        files = await alist.list_dir("/test/path")
        assert files == ["file1.txt", "file2.txt"]
        mock_api_call.assert_called_once_with(
            "POST",
            "api/fs/list",
            json={
                "path": "/test/path",
                "password": None,
                "page": 1,
                "per_page": 30,
                "refresh": False,
            },
        )


@pytest.mark.asyncio
async def test_get_task_list(alist):
    with patch.object(
        alist, "_fetch_tasks", new_callable=AsyncMock
    ) as mock_fetch_tasks:
        mock_fetch_tasks.return_value = AlistTaskCollection(
            [
                AlistDownloadTask(
                    tid="task1",
                    description="test task",
                    status=AlistTaskStatus.Running,
                    progress=0.5,
                )
            ]
        )
        tasks = await alist.get_task_list(AlistTaskType.DOWNLOAD, AlistTaskState.UNDONE)
        assert isinstance(tasks, AlistTaskCollection)
        assert len(tasks) == 1
        assert tasks[0].tid == "task1"
        mock_fetch_tasks.assert_called_once_with(
            AlistTaskType.DOWNLOAD, AlistTaskState.UNDONE
        )


@pytest.mark.asyncio
async def test_cancel_task(alist):
    with patch.object(alist, "_api_call", new_callable=AsyncMock) as mock_api_call:
        task = AlistDownloadTask(
            tid="task1",
            description="test task",
            status=AlistTaskStatus.Running,
            progress=0.5,
        )
        result = await alist.cancel_task(task)
        assert result is True
        mock_api_call.assert_called_once_with(
            "POST", "/api/admin/task/offline_download/cancel?tid=task1"
        )


@pytest.mark.asyncio
async def test_rename(alist):
    with patch.object(alist, "_api_call", new_callable=AsyncMock) as mock_api_call:
        result = await alist.rename("/old/path/file.txt", "new_file.txt")
        assert result is True
        mock_api_call.assert_called_once_with(
            "POST",
            "api/fs/rename",
            json={"path": "/old/path/file.txt", "name": "new_file.txt"},
        )


def test_alist_task_collection():
    task1 = AlistDownloadTask(
        tid="task1",
        description="test task 1",
        status=AlistTaskStatus.Running,
        progress=0.5,
    )
    task2 = AlistDownloadTask(
        tid="task2",
        description="test task 2",
        status=AlistTaskStatus.Pending,
        progress=0.0,
    )

    collection = AlistTaskCollection([task1, task2])

    assert len(collection) == 2
    assert collection["task1"] == task1
    assert collection[0] == task1
    assert task1 in collection

    task3 = AlistDownloadTask(
        tid="task3",
        description="test task 3",
        status=AlistTaskStatus.Succeeded,
        progress=1.0,
    )
    collection.add_task(task3)

    assert len(collection) == 3
    assert collection["task3"] == task3

    new_collection = AlistTaskCollection([task1])
    combined_collection = collection + new_collection

    assert len(combined_collection) == 3
    assert combined_collection["task1"] == task1
    assert combined_collection["task2"] == task2
    assert combined_collection["task3"] == task3


def test_alist_download_task():
    task = AlistDownloadTask.from_json(
        {
            "id": "task1",
            "name": "download https://example.com/file.zip to /path/to/save",
            "state": 1,
            "progress": 0.5,
            "error": None,
        }
    )

    assert task.tid == "task1"
    assert task.description == "download https://example.com/file.zip to /path/to/save"
    assert task.status == AlistTaskStatus.Running
    assert math.isclose(task.progress, 0.5, rel_tol=1e-9, abs_tol=1e-9)
    assert task.error_msg is None
    assert task.url == "https://example.com/file.zip"
    assert task.task_type == AlistTaskType.DOWNLOAD


def test_alist_transfer_task():
    task = AlistTransferTask.from_json(
        {
            "id": "task1",
            "name": "transfer /download/123456/file.zip to [/path/to/save]",
            "state": 2,
            "progress": 1.0,
            "error": None,
        }
    )

    assert task.tid == "task1"
    assert task.description == "transfer /download/123456/file.zip to [/path/to/save]"
    assert task.status == AlistTaskStatus.Succeeded
    assert math.isclose(task.progress, 1.0, rel_tol=1e-9, abs_tol=1e-9)
    assert task.error_msg is None
    assert task.uuid == "123456"
    assert task.file_name == "file.zip"
    assert task.task_type == AlistTaskType.TRANSFER
